@@ -2,6 +2,6 @@ from .invenio_base import InvenioBaseClassPythonBuilder
 
 
 class InvenioRecordMetadataBuilder(InvenioBaseClassPythonBuilder):
-    TYPE = 'invenio_record_metadata'
-    class_config = 'record-metadata-class'
-    template = 'record-metadata'
+    TYPE = "invenio_record_metadata"
+    section = "record-metadata"
+    template = "record-metadata"
